require File.join(File.dirname(__FILE__), 'lib/certapi')
require File.join(File.dirname(__FILE__), 'lib/certificate_logger')
require File.join(File.dirname(__FILE__), 'lib/certificate_helpers')

module Certificates
  class Main < Sinatra::Base
    set :root, $path

    helpers CertificateHelpers

    configure do
      dir = '/opt/puppet/share/puppet-dashboard'

      settings = YAML::load(IO.read(File.join(dir, 'config', 'settings.yml')))

      CertAPI.config({
        :ca_location    => settings['ca_server'],
        :ca_port        => settings['ca_port'],
        :ca_file_path   => File.join(dir, settings['ca_certificate_path']),
        :cert_path      => File.join(dir, settings['certificate_path']),
        :cert_key_path  => File.join(dir, settings['private_key_path'])
      })

      mime_type :csr,
        "application/vnd.puppetlabs.certificate-signing-request.metadata.v1+json"
    end

    before do
      cache_control :no_cache
      @logger = CertificateLogger.new
    end

    get '/api/v1/csrs' do
      content_type :csr
      real_and_cached_csrs.to_json
    end

    get '/api/v1/csr_count' do
      content_type :json
      {:request_count => get_csrs.count}.to_json
    end

    get '/api/v1/csrs/:certname' do
      certname = params[:certname]
      unless csr_exists?(certname)
        halt 404, "#{certname} is not a valid certname. " +
                  "The valid certnames are #{CertAPI.certnames.to_json}"
        # It is okay to divulge a list of certnames in the error message, as
        # the user accessing this information must be authenticated to reach
        # this API. If they have permission access to the API, then they have
        # access to this information anyway.
      else
        content_type :csr
        CertAPI.cert_info(certname).to_json
      end
    end

    # sign a CSR
    put '/api/v1/csrs/:certname' do
      certname = params[:certname]
      unless csr_exists?(certname)
        halt 403, "#{certname} could not be accepted because its certificate " +
                  "request has been moved or modified."
        # This status code could be 403 or 404, but since we are providing the
        # reason for the error it is better left as 403.
      else
        single_sign(certname, @logger).code
      end
    end

    # sign multiple CSRs
    post '/api/v1/csrs/multi_accept' do
      content_type :json
      multi_sign(params[:names], @logger).to_json
    end

    # reject a CSR
    delete '/api/v1/csrs/:certname' do
      certname = params[:certname]
      unless csr_exists?(certname)
        halt 403, "#{certname} could not be rejected because its certificate " +
                  "request has been moved or modified."
      else
        single_reject(certname, @logger).code
      end
    end

    # reject multiple CSRs
    post '/api/v1/csrs/multi_reject' do
      content_type :json
      multi_reject(params[:names], @logger).to_json
    end

    # Remove the actioned (approved/rejected) cert from the cache
    post '/api/v1/cache/multi_purge' do
      multi_purge(params[:names], @logger).to_json
    end
  end
end
