define(['underscore', 'jquery', 'sandbox', '../models/request'],
  function(_, $, sandbox, RequestModel) {

  var RequestsCollection = sandbox.mvc.Collection({

    url: 'services/certificates/api/v1/csrs',

    model: RequestModel,

    /**
     * Return a list of requests that have been accepted or rejected.
     * @return {Array}
     */
    completed_requests: function() {
      return this.where({approvable: true, request_open: false});
    },

    /**
     * Return a list of requests that are currently waiting to be accepted or rejected.
     * Note that this also includes requests with DNS alt names, which are not actionable via
     * the certificate manager.
     * @return {Array}
     */
    open_requests: function() {
      return this.where({status: 'pending'})
    },

    /**
     * Return a list of requests that can be accepted or rejected through the GUI.
     * @return {Array}
     */
    actionable_requests: function() {
      return this.where({approvable: true, request_open: true});
    },

    /**
     * Destroy the model instances of all requests that have been actioned.
     */
    clear_requests: function() {
      var names = [];

      _.each(this.completed_requests(), function(model) {
        names.push(model.get('name'));

        model.destroy();
      });

      $.ajax({
        type: 'POST',
        url: 'services/certificates/api/v1/cache/multi_purge',
        data: {names: names}
      });
    },

    /**
     * Accept all open and actionable requests
     * @return {jQuery.promise}
     */
    accept_all: function() {
      return this._do_multi_request('accept', function(model) {
                model.set_accepted();
              });

    },

    /**
     * Reject all open and actionable requests
     * @return {jQuery.promise}
     */
    reject_all: function() {
      return this._do_multi_request('reject', function(model) {
                model.set_rejected();
              });

    },

    /**
     * @param action string
     * @param model_callback function
     * @return {jQuery.promise}
     * @private
     */
    _do_multi_request: function(action, model_callback) {

      // getting the actionable requests here means we'll always be consistent
      // and call the callback on the same list of requests that we send to the server
      var actionable = this.actionable_requests(),
          deferred = new sandbox.data.deferred();

      if (actionable.length === 0) {
        deferred.resolve();

        return deferred.promise();
      }

      var that = this,
          names = [];

      _.each(actionable, function(model) {
        names.push(model.get('name'));

        // notify the request that its status is being updated on the server
        model.set_sync_saving();
      });

      var url = 'services/certificates/api/v1/csrs/multi_' + action;

      deferred.done(function(response) {

        _.each(response, function(ret, certname) {

          var model = that.get(certname);

          if (ret['result'] == 'success') {

            model_callback(model);
            model.set_sync_complete();

          } else {

            model.set_failed(ret['message']);
            model.set_sync_failed();
          }
        });

      });

      deferred.fail(function() {
        _.each(names, function(certname) {
          var model = that.get(certname);

          // we don't set_sync_failed here as we don't have an error message that can be shown for
          // each individual request - the entire ajax request failed
          model.set_sync_complete();
        });
      });

      $.ajax({
        type: 'POST',

        url: url,

        data: {names: names},

        success: function(response) {
          deferred.resolve(response);
        },

        error: function(response) {
          deferred.reject(response.responseText);
        }
      });

      return deferred.promise();
    },

    parse: function(resp) {
      var ret = [];
      var num_nodes = resp.length;
      for (var i = 0; i < num_nodes; i++) {
        var certname = resp[i]['name'];
        var fingerprint = resp[i]['fingerprints']['MD5'];
        var altnames = resp[i]['dns_alt_names'];
        var has_altname = (altnames.length != 0);
        var state = resp[i]['state'];
        var status = null;
        if (state === 'signed') {
          status = 'accepted';
        } else if (state === 'rejected') {
          status = 'rejected';
        }

        obj = {
          name: certname,
          fingerprint: fingerprint,
          dns_alt_name: altnames,
          has_altname: has_altname,
          approvable: !has_altname,
          request_open: status == null && !has_altname,
        };

        if (status) {
          obj['status'] = status;
        }

        ret.push(obj);
      }

      return ret;
    }
  });

  return RequestsCollection;

});
