
define(['../../../collections/requests.js', '../../../models/request.js'],
  function(RequestsCollection, RequestModel) {

  describe('Request collection', function() {
    var collection,
        models;

    beforeEach(function() {
      models = {
        'fake1': new RequestModel({
          name: 'fake1',
          approvable: true,
          request_open: true,
          has_altname: false
        }),
        'fake2': new RequestModel({
          name: 'fake2',
          approvable: true,
          request_open: true,
          has_altname: false
        }),
        'altname1': new RequestModel({
          name: 'altname1',
          approvable: false,
          request_open: false,
          has_altname: true
        })
      };

      collection = new RequestsCollection;

      collection.add([models.fake1, models.fake2, models.altname1]);
    });

    it('only returns actionable requests when asked', function() {
      var requests = collection.actionable_requests();

      expect(requests).toEqual([models.fake1, models.fake2]);
    });

    it('only returns open requests when asked', function() {
      var requests = collection.open_requests();

      expect(requests).toEqual([models.fake1, models.fake2, models.altname1]);
    });

    it('only returns approvable completed requests when asked', function() {
      models.fake1.set('request_open', false);
      models.fake2.set('request_open', false);

      var requests = collection.completed_requests();

      expect(requests).toEqual([models.fake1, models.fake2]);
    });

    describe('clearing actioned requests', function() {
      var server;

      beforeEach(function() {
        server = sinon.fakeServer.create();

        models.fake1.set('request_open', false);
        models.fake1.set('approvable', true);

        models.fake2.set('request_open', false);
        models.fake2.set('approvable', true);
      });

      afterEach(function() {
        server.restore();
      });

      it('destroys all completed models when clearing requests', function() {

        var destroy1 = sinon.spy(models.fake1, 'destroy'),
            destroy2 = sinon.spy(models.fake2, 'destroy'),
            destroy3 = sinon.spy(models.altname1, 'destroy');

        collection.clear_requests();

        expect(collection.models).toEqual([models.altname1]);

        expect(destroy1).toHaveBeenCalledOnce();
        expect(destroy2).toHaveBeenCalledOnce();
        expect(destroy3).not.toHaveBeenCalled();
      });

      it('should tell the server to purge the models from the cache', function() {
        collection.clear_requests();

        expect(server.requests[0].url).toBe('services/certificates/api/v1/cache/multi_purge');
        expect(server.requests[0].method).toBe('POST');
      });
    });

    xit('parses server responses in to models', function() {
      // deferred until the API rationalisation
    });

    describe('actioning requests', function() {

      var server;

      beforeEach(function() {
        server = sinon.fakeServer.create();
      });

      afterEach(function() {
        server.restore();
      });

      _.each(['accept', 'reject'], function(action) {

        var method = action + '_all',
            actioned = action + 'ed';

        describe(action + 'ing all', function() {
          it('has the correct URL and HTTP method', function() {
            collection[method]();

            expect(server.requests[0].url).toBe('services/certificates/api/v1/csrs/multi_' + action);
            expect(server.requests[0].method).toBe('POST');
          });

          it('calls callbacks with the entire response on success', function() {
            var spy = sinon.spy();

            var payload = {
              fake1: {
                result: 'success'
              },
              fake2: {
                result: 'success'
              }
            };

            server.respondWith([200, {"Content-Type": "application/json"}, JSON.stringify(payload)]);

            collection[method]().done(spy);

            server.respond();

            expect(spy).toHaveBeenCalledWith(payload);
          });

          it('calls callbacks with the response text on failure', function() {
            var spy = sinon.spy();

            server.respondWith([500, {}, 'An error occured']);

            collection[method]().fail(spy);

            server.respond();

            expect(spy).toHaveBeenCalledWith('An error occured');
          });

          it('sets all models to "saving" sync status', function() {
            var syncing1 = sinon.mock(models.fake1).expects('set_sync_saving').once(),
                syncing2 = sinon.mock(models.fake2).expects('set_sync_saving').once();

            collection[method]();

            syncing1.verify();
            syncing2.verify();
          });

          it(action + 's all open models on success', function() {
            var set_actioned1 = sinon.mock(models.fake1).expects('set_' + actioned).once(),
                set_actioned2 = sinon.mock(models.fake2).expects('set_' + actioned).once(),
                set_complete1 = sinon.mock(models.fake1).expects('set_sync_complete').once(),
                set_complete2 = sinon.mock(models.fake2).expects('set_sync_complete').once();

            server.respondWith([200, {"Content-Type": "application/json"},
                               '{"fake1": {"result":"success"}, "fake2": {"result":"success"}}']);

            collection[method]();

            server.respond();

            set_actioned1.verify();
            set_actioned2.verify();

            set_complete1.verify();
            set_complete2.verify();
          });

          it('returns information on any CSRs it failed to sign', function() {
            var set_failed1 = sinon.mock(models.fake1).expects('set_failed').once().withExactArgs('Something happened!'),
                set_sync_failed1 = sinon.mock(models.fake1).expects('set_sync_failed').once();

            server.respondWith([200, {"Content-Type": "application/json"},
                               '{"fake1": {"result":"fail", "message":"Something happened!"}}']);

            collection[method]();

            server.respond();

            set_failed1.verify();
            set_sync_failed1.verify();
          });

          it('resolves the promise if there are no open models', function() {
            var callback = sinon.spy();

            models.fake1.set('request_open', false);
            models.fake2.set('request_open', false);

            collection[method]().done(callback);

            expect(server.requests.length).toBe(0);
            expect(callback).toHaveBeenCalledOnce();
          });

          it('completes the sync of a model if the entire request fails', function() {
            var set_complete1 = sinon.mock(models.fake1).expects('set_sync_complete').once(),
                set_complete2 = sinon.mock(models.fake2).expects('set_sync_complete').once();

            server.respondWith([500, {}, 'An error occured']);

            collection[method]()

            server.respond();

            set_complete1.verify();
            set_complete2.verify();
          });
        });
      });
    });
  });

});
